@@ -3,6 +3,6 @@
 
 arma::mat outliersC(const arma::colvec &z, bool bc, double mu, const arma::colvec &phi,
                     const arma::colvec &nabla, const arma::colvec &theta, 
-                    arma::ucolvec &timing, double c);  
+                    arma::ucolvec &timing, bool eres, double c);  
 
 #endif //__ARIMA_OUTLIERS__
